// AGG Info

import com.certicom.ecc.jcae.Certicom;
import com.certicom.ecc.system.SystemConfig;
import com.macrovision.flexlm.FlexlmConstants;
import java.io.*;
import java.security.Provider;
import java.security.Security;
import java.util.*;

public class CreateVendorInfo
    implements FlexlmConstants
{

    public static void main(String args[])
    {
        (new CreateVendorInfo()).doMain(args);
    }

    public CreateVendorInfo()
    {
        subClassName = "MyVendorInfo";
        outDir = null;
        packageName = null;
        includeNonCroSeeds = false;
        lmcodePath = null;
        pubkeyPath = null;
        lmseedsPath = null;
        vendorItems = new HashMap();
    }

    public void doMain(String as[])
    {
        parseArgs(as);
        initCerticom();
        createVendorInfoFile();
        exit(0);
    }

    public String usage()
    {
        String s = System.getProperty("line.separator");
        return "usage: java " + getClass().getName() + " [options]" + s + "-classname classname   specify class name for output" + s + "-code lm_code.h        specific full path to lm_code.h" + s + "-include-non-cro-seeds include SEED1 and SEED2" + s + "-outdir directory      specify output directory" + s + "-packagename package   specify package name for output" + s + "-pubkey lmpubkey.h     specify full path to lmpubkey.h" + s + "-seeds lmseeds.h       specific full path to lmseeds.h";
    }

    public void parseArgs(String as[])
    {
        for(int i = 0; i < as.length; i++)
            if(as[i].equals("-classname"))
                subClassName = as[++i];
            else
            if(as[i].equals("-code"))
                lmcodePath = as[++i];
            else
            if(as[i].equals("-include-non-cro-seeds"))
                includeNonCroSeeds = true;
            else
            if(as[i].equals("-outdir"))
                outDir = as[++i];
            else
            if(as[i].equals("-packagename"))
                packageName = as[++i];
            else
            if(as[i].equals("-pubkey"))
                pubkeyPath = as[++i];
            else
            if(as[i].equals("-seeds"))
            {
                lmseedsPath = as[++i];
            } else
            {
                String s = "Unknown command line argument '" + as[i] + "'" + System.getProperty("line.separator") + usage();
                fatal(s);
            }

        if(lmcodePath == null && lmseedsPath == null)
            fatal("No lm_code.h or lmseeds.h filename specified (use -code or -seeds)");
        if(pubkeyPath == null && !includeNonCroSeeds)
        {
            String s1 = "You must specify at least one of -pubkey or -include-non-cro-seeds";
            fatal(s1);
        }
    }

    public void initCerticom()
    {
        Certicom certicom = new Certicom();
        if(Security.addProvider(certicom) < 0)
        {
            throw new RuntimeException("Can't init Certicom");
        } else
        {
            providerName = certicom.getName();
            SystemConfig systemconfig = SystemConfig.getConfig();
            systemconfig.setConformance(16);
            systemconfig.setPtCompression(2);
            return;
        }
    }

    public void setVendorItem(String s, String s1)
    {
        vendorItems.put(s, s1);
    }

    public String getVendorItem(String s)
    {
        return (String)vendorItems.get(s);
    }

    public String getVendorWord(String s)
    {
        String s1 = getVendorItem(s);
        if(s1 == null)
            return null;
        StringTokenizer stringtokenizer = new StringTokenizer(s1);
        if(!stringtokenizer.hasMoreTokens())
            return "";
        else
            return stringtokenizer.nextToken();
    }

    protected void createVendorInfoFile()
    {
        readLmcodeFile(lmcodePath);
        if(lmseedsPath != null)
            readLmcodeFile(lmseedsPath);
        if(pubkeyPath != null)
            readLmpubkeyFile();
        generateVendorInfoSubClass();
    }

    protected void readLmcodeFile(String s)
    {
        LineNumberReader linenumberreader = null;
        try
        {
            linenumberreader = new LineNumberReader(new FileReader(s));
        }
        catch(FileNotFoundException filenotfoundexception)
        {
            fatal("Input file " + s + " not found: " + filenotfoundexception.getMessage());
        }
        try
        {
            readLmcodeFile(linenumberreader);
        }
        catch(IOException ioexception)
        {
            fatal("Error reading input file " + s + ": " + ioexception.getMessage());
        }
        try
        {
            linenumberreader.close();
        }
        catch(IOException ioexception1)
        {
            fatal("Error closing input file " + s + ": " + ioexception1.getMessage());
        }
    }

    protected void readLmpubkeyFile()
    {
        LineNumberReader linenumberreader = null;
        try
        {
            linenumberreader = new LineNumberReader(new FileReader(pubkeyPath));
        }
        catch(FileNotFoundException filenotfoundexception)
        {
            fatal("Input file " + pubkeyPath + " not found: " + filenotfoundexception.getMessage());
        }
        try
        {
            readLmpubkeyFile(linenumberreader);
        }
        catch(IOException ioexception)
        {
            fatal("Error reading input file " + pubkeyPath + ": " + ioexception.getMessage());
        }
        try
        {
            linenumberreader.close();
        }
        catch(IOException ioexception1)
        {
            fatal("Error closing input file " + pubkeyPath + ": " + ioexception1.getMessage());
        }
    }

    protected void readLmcodeFile(LineNumberReader linenumberreader)
        throws IOException
    {
        String s;
        while((s = readContinuedLine(linenumberreader)) != null) 
            if(s.startsWith("#"))
            {
                s = s.substring(1).trim();
                if(s.startsWith("define"))
                {
                    s = s.substring(6).trim();
                    StringTokenizer stringtokenizer = new StringTokenizer(s);
                    if(stringtokenizer.countTokens() >= 2)
                    {
                        String s1 = stringtokenizer.nextToken();
                        String s2 = stringtokenizer.nextToken();
                        setVendorItem(s1, s2);
                    }
                }
            }
    }

    void checkRequiredVendorItems()
    {
        String as[] = {
            "VENDOR_NAME", "VENDOR_KEY1", "VENDOR_KEY2", "VENDOR_KEY3", "VENDOR_KEY4", "VENDOR_KEY5", "ENCRYPTION_SEED1", "ENCRYPTION_SEED2"
        };
        int i = 0;
        for(int j = 0; j < as.length; j++)
            if(getVendorItem(as[j]) == null)
            {
                println("Missing lm_code.h/lmseeds.h item " + as[j]);
                i++;
            }

        if(i > 0)
            exit(1);
    }

    protected void readLmpubkeyFile(LineNumberReader linenumberreader)
        throws IOException
    {
        int i = -1;
        boolean flag = false;
        int ai[] = null;
        int ai1[][] = new int[3][];
        String s;
        while((s = readContinuedLine(linenumberreader)) != null) 
        {
            s = s.trim();
            int j = s.lastIndexOf("{");
            int l = s.indexOf("}");
            if(j >= 0 && l >= 0)
            {
                String s1 = s.substring(j + 1, l);
                String s2 = extractVariableName(s);
                if(s2 != null)
                {
                    if(s2.equals("lm_pubkey"))
                        i = 0;
                    else
                        i = -1;
                    if(s2.equals("lm_pubsize"))
                        flag = true;
                    else
                        flag = false;
                }
    
                if(flag)
                    ai = parseHexList(s1);
                    
                if(i >= 0 && i <= 2)
                    ai1[i++] = parseHexList(s1);
            }
        }
        if(ai == null)
            fatal("No lm_pubsize list found");
        if(ai.length != 3)
            fatal("Bad length for lm_pubsize, must be 3");
        if(ai1[0] == null)
            fatal("No public keys found");
        if(ai1[1] == null || ai1[2] == null)
            fatal("Too few public keys found");
        publicKeys = new byte[3][];
        for(int k = 0; k < 3; k++)
        {
            int i1 = ai[k];
            if(i1 > ai1[k].length)
                fatal("Too few bytes specified in public key " + k);
            publicKeys[k] = new byte[i1];
            for(int j1 = 0; j1 < i1; j1++)
                publicKeys[k][j1] = (byte)ai1[k][j1];

        }

    }

    public String extractVariableName(String s)
    {
        int i = s.indexOf("[");
        if(i <= 0)
            return null;
        for(s = s.substring(0, i); s.endsWith(" "); s = s.substring(0, s.length() - 1));
        int j = s.lastIndexOf(" ");
        return s.substring(j + 1);
    }

    public int[] parseHexList(String s)
    {
        StringTokenizer stringtokenizer = new StringTokenizer(s, " \t,");
        Vector vector = new Vector();
        int i;
        for(; stringtokenizer.hasMoreTokens(); vector.addElement(new Integer(i)))
        {
            String s1 = stringtokenizer.nextToken();
            i = (int)parseHexString(s1);
        }

        int ai[] = new int[vector.size()];
        for(int j = 0; j < ai.length; j++)
            ai[j] = ((Integer)vector.elementAt(j)).intValue();

        return ai;
    }

    protected String readContinuedLine(LineNumberReader linenumberreader)
        throws IOException
    {
        String s = readLineStripLineSeparator(linenumberreader);
        if(s == null)
            return null;
        if(!s.endsWith("\\"))
            return s;
        StringBuffer stringbuffer = new StringBuffer(s);
        boolean flag = true;
        while(flag) 
        {
            String s1 = readLineStripLineSeparator(linenumberreader);
            if(s1 == null)
                break;
            flag = s1.endsWith("\\");
            if(flag)
                s1 = s1.substring(0, s1.length() - 1);
            stringbuffer.append(s1);
        }
        return stringbuffer.toString();
    }

    protected String readLineStripLineSeparator(LineNumberReader linenumberreader)
        throws IOException
    {
        String s = linenumberreader.readLine();
        if(s == null)
            return null;
        if(s.endsWith("\r\n"))
            s = s.substring(0, s.length() - 2);
        else
        if(s.endsWith("\r") || s.endsWith("\n"))
            s = s.substring(0, s.length() - 1);
        return s;
    }

    protected void generateVendorInfoSubClass()
    {
        String s = subClassName + ".java";
        if(outDir != null)
            s = outDir + File.separator + s;
        PrintWriter printwriter = null;
        try
        {
            printwriter = new PrintWriter(new FileWriter(s));
        }
        catch(IOException ioexception)
        {
            println("Error opening output file " + s + ": " + ioexception.getMessage());
            exit(1);
        }
        printwriter.println("//" + s);
        generateHeader(printwriter);
        generateConstructor(printwriter);
        generateGetVendorName(printwriter);
        generateGetPublicKey(printwriter);
        generateGetEncryptionSeeds(printwriter);
        generateGetVendorKeys(printwriter);
        generateGetCroKeys(printwriter);
        generateGetDefaultStrength(printwriter);
        generateFooter(printwriter);
        printwriter.close();
    }

    protected void generateHeader(PrintWriter printwriter)
    {
        printwriter.println("//This file is auto-generated, do not edit by hand");
        printwriter.println();
        if(packageName != null && !packageName.equals(""))
        {
            printwriter.println("package " + packageName + ";");
            printwriter.println();
        }
        printwriter.println("import com.macrovision.flexlm.FlexlmConstants;");
        printwriter.println("import com.macrovision.flexlm.FlexlmException;");
        printwriter.println("import com.macrovision.flexlm.VendorInfo;");
        printwriter.println("import com.macrovision.flexlm.misc.FlexlmPublicKey;");
        printwriter.println("import java.security.PublicKey;");
        printwriter.println();
        printwriter.print("public class ");
        printwriter.print(subClassName);
        printwriter.println(" extends VendorInfo implements FlexlmConstants {");
    }

    protected void generateConstructor(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic " + subClassName + "() throws FlexlmException {");
        printwriter.println("\t}");
    }

    protected void generateFooter(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("}");
        printwriter.println();
        printwriter.println("//end");
    }

    protected void generateGetVendorName(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic String getVendorName() {");
        printwriter.print("\t\treturn \"");
        String s = getVendorWord("VENDOR_NAME");
        if(s.startsWith("\""))
            s = s.substring(1);
        if(s.endsWith("\""))
            s = s.substring(0, s.length() - 1);
        printwriter.print(s);
        printwriter.println("\";");
        printwriter.println("\t}");
    }

    protected void generateGetPublicKey(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic PublicKey getPublicKey(int strength) {");
        if(publicKeys == null)
        {
            printwriter.println("\t\tthrow new RuntimeException(\"No public keys available\");");
            printwriter.println("\t}");
            return;
        } else
        {
            printwriter.println("\t\tswitch (strength) {");
            printwriter.println("\t\tcase LM_STRENGTH_113BIT:");
            printwriter.print("\t\treturn new FlexlmPublicKey(getVendorName(),\"");
            String s = getPublicKeyString(2);
            printwriter.print(s);
            printwriter.println("\");\n");
            printwriter.println("\t\tcase LM_STRENGTH_163BIT:");
            printwriter.print("\t\treturn new FlexlmPublicKey(getVendorName(),\"");
            s = getPublicKeyString(3);
            printwriter.print(s);
            printwriter.println("\");\n");
            printwriter.println("\t\tcase LM_STRENGTH_239BIT:");
            printwriter.print("\t\treturn new FlexlmPublicKey(getVendorName(),\"");
            s = getPublicKeyString(4);
            printwriter.print(s);
            printwriter.println("\");\n");
            printwriter.println("\t\tdefault:");
            printwriter.println("\t\t\treturn null;");
            printwriter.println("\t\t}");
            printwriter.println("\t}");
            return;
        }
    }

    protected void generateGetEncryptionSeeds(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic int[] getEncryptionSeeds() {");
        printwriter.print("\t\treturn new int[] {");
        long l = parseHexItem("ENCRYPTION_SEED1");
        long l1 = parseHexItem("ENCRYPTION_SEED2");
        long l2 = parseHexItem("VENDOR_KEY5");
        if(includeNonCroSeeds)
        {
            printwriter.print("0x" + Long.toString(l ^ l2, 16));
            printwriter.print(",");
            printwriter.print("0x" + Long.toString(l1 ^ l2, 16));
        } else
        {
            long l3 = parseHexItem("VENDOR_KEY1");
            long l4 = parseHexItem("VENDOR_KEY2");
            long l5 = l3 ^ l4;
            printwriter.print("0x" + Long.toString(l5, 16));
            printwriter.print(",");
            long l6 = l ^ l1 ^ l5;
            printwriter.print("0x" + Long.toString(l6, 16));
        }
        printwriter.println(" };");
        printwriter.println("\t}");
    }

    protected void generateGetVendorKeys(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic int[] getVendorKeys() {");
        printwriter.print("\t\treturn new int[] {");
        printwriter.print(getVendorWord("VENDOR_KEY1"));
        printwriter.print(",");
        printwriter.print(getVendorWord("VENDOR_KEY2"));
        printwriter.print(",");
        printwriter.print(getVendorWord("VENDOR_KEY3"));
        printwriter.print(",");
        printwriter.print(getVendorWord("VENDOR_KEY4"));
        printwriter.println(" };");
        printwriter.println("\t}");
    }

    protected void generateGetCroKeys(PrintWriter printwriter)
    {
        printwriter.println();
        printwriter.println("\tpublic int[] getCroKeys() {");
        printwriter.print("\t\treturn new int[] {");
        long l = parseHexItem("CRO_KEY1");
        long l1 = parseHexItem("CRO_KEY2");
        if(l != 0L && publicKeys != null)
        {
            printwriter.print("0x" + Long.toString(l, 16));
            printwriter.print(",");
            printwriter.print("0x" + Long.toString(l1, 16));
        }
        printwriter.println(" };");
        printwriter.println("\t}");
    }

    protected void generateGetDefaultStrength(PrintWriter printwriter)
    {
        String s = getVendorWord("LM_STRENGTH");
        if(s == null)
            s = "LM_STRENGTH_DEFAULT";
        printwriter.println();
        printwriter.println("\tpublic int getDefaultStrength() {");
        printwriter.println("\t\treturn " + s + ";");
        printwriter.println("\t}");
    }

    public String getPublicKeyString(int i)
    {
        byte abyte0[] = null;
        switch(i)
        {
        case 2: // '\002'
            abyte0 = publicKeys[0];
            break;

        case 3: // '\003'
            abyte0 = publicKeys[1];
            break;

        case 4: // '\004'
            abyte0 = publicKeys[2];
            break;

        default:
            fatal("Bad strength " + i + " in getPublicKeyString");
            break;
        }
        return toHexString(abyte0);
    }

    public long parseHexItem(String s)
    {
        String s1 = getVendorWord(s);
        if(s1 == null || s1.length() == 0)
            return 0L;
        else
            return parseHexString(s1);
    }

    public long parseHexString(String s)
    {
        if(s == null)
            return 0L;
        if(s.startsWith("0x") || s.startsWith("0X"))
            s = s.substring(2);
        long l = Long.parseLong(s, 16);
        return l;
    }

    public String toHexString(byte abyte0[])
    {
        StringBuffer stringbuffer = new StringBuffer();
        String s = "0123456789ABCDEF";
        for(int i = 0; i < abyte0.length; i++)
        {
            int j = abyte0[i] >> 4 & 0xf;
            int k = abyte0[i] & 0xf;
            stringbuffer.append(s.charAt(j));
            stringbuffer.append(s.charAt(k));
        }

        return stringbuffer.toString();
    }

    public void exit(int i)
    {
        System.exit(i);
    }

    public void println(String s)
    {
        System.out.println(s);
    }

    public void fatal(String s)
    {
        println(s);
        exit(1);
    }

    protected String providerName;
    protected String subClassName;
    protected String outDir;
    protected String packageName;
    protected boolean includeNonCroSeeds;
    protected Map vendorItems;
    protected String lmcodePath;
    protected String pubkeyPath;
    protected String lmseedsPath;
    protected byte publicKeys[][];
}
